use cgmath::{Point3, Vector2};
use galactica_render::{ObjectSprite, StarfieldStar};
use rand::{self, Rng};

use super::SystemObject;
use crate::{consts, content};

pub struct System {
	pub name: String,
	bodies: Vec<SystemObject>,
	pub starfield: Vec<StarfieldStar>,
}

impl System {
	pub fn new(ct: &content::System) -> Self {
		let mut rng = rand::thread_rng();
		let sz = consts::STARFIELD_SIZE as f32 / 2.0;
		let mut s = System {
			name: ct.name.clone(),
			bodies: Vec::new(),
			starfield: (0..consts::STARFIELD_COUNT)
				.map(|_| StarfieldStar {
					pos: Point3 {
						x: rng.gen_range(-sz..=sz),
						y: rng.gen_range(-sz..=sz),
						z: rng.gen_range(consts::STARFIELD_Z_MIN..consts::STARFIELD_Z_MAX),
					},
					size: rng.gen_range(consts::STARFIELD_SIZE_MIN..consts::STARFIELD_SIZE_MAX),
					tint: Vector2 {
						x: rng.gen_range(0.0..=1.0),
						y: rng.gen_range(0.0..=1.0),
					},
				})
				.collect(),
		};

		for o in &ct.objects {
			s.bodies.push(SystemObject {
				pos: o.position,
				sprite_texture: o.sprite_texture,
				size: o.size,
				angle: o.angle,
			});
		}

		return s;
	}

	pub fn get_sprites(&self) -> Vec<ObjectSprite> {
		return self.bodies.iter().map(|x| x.get_sprite()).collect();
	}
}
